@@ -755,6 +755,20 @@ impl<'app> Builder<'app> {
             }
         }
 
+        // Set the class type for X11 if WindowExtUnix trait is compiled in winit
+        // (see lines https://docs.rs/winit/0.26.0/src/winit/platform/unix.rs.html#1-7)
+        #[cfg(any(
+            target_os = "linux",
+            target_os = "dragonfly",
+            target_os = "freebsd",
+            target_os = "netbsd",
+            target_os = "openbsd"
+        ))]
+        {
+            use winit::platform::unix::WindowBuilderExtUnix;
+            window = window.with_class("nannou".to_string(), "nannou".to_string());
+        }
+
         // Set default dimensions in the case that none were given.
         let initial_window_size = window
             .window
